/*
**  NSRegEx.m
**
**  Copyright (c) 2001, 2002
**
**  Author: Francis Lachapelle <francis@Sophos.ca>
**          Ludovic Marcotte <ludovic@Sophos.ca>
**
**  This library is free software; you can redistribute it and/or
**  modify it under the terms of the GNU Lesser General Public
**  License as published by the Free Software Foundation; either
**  version 2.1 of the License, or (at your option) any later version.
**  
**  This library is distributed in the hope that it will be useful,
**  but WITHOUT ANY WARRANTY; without even the implied warranty of
**  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU
**  Lesser General Public License for more details.
**  
**  You should have received a copy of the GNU Lesser General Public
**  License along with this library; if not, write to the Free Software
**  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
*/

#include <Pantomime/NSRegEx.h>

#include <Pantomime/Constants.h>

#import <Foundation/NSArray.h>
#import <Foundation/NSDebug.h>
#import <Foundation/NSValue.h>

#include <sys/types.h>
#include <regex.h>
#include <stdlib.h>

@implementation NSRegEx 

+ (NSArray *) matchString: (NSString *) theString
	      withPattern: (NSString *) thePattern
	  isCaseSensitive: (BOOL) caseSensitive
{
  NSMutableArray *aMutableArray;

  int flags, offset, status;
  char *s, *error;

  regmatch_t rm[1];
  regex_t re;

  flags = REG_EXTENDED;

  if ( !caseSensitive )
    {
      flags |= REG_ICASE;
    }

  if (regcomp(&re, [thePattern cString], flags) != 0)
    {
      return [NSArray array];
    }

  s = (char*)[theString lossyCString];
  aMutableArray = [[NSMutableArray alloc] init];
  
  status = regexec(&re, s, 1, rm, 0);
  offset = 0;
 
  while (status == 0)
    {
      NSValue *aValue;

      aValue = [NSValue valueWithRange: NSMakeRange(offset + rm[0].rm_so,  rm[0].rm_eo - rm[0].rm_so)];
      
      [aMutableArray addObject: aValue];
      
      offset += rm[0].rm_eo;
      status = regexec(&re, s + offset, 1, rm, REG_NOTBOL);
    }

  if (status != 0) 
    {
      error = malloc(255*sizeof(char));
      regerror(status, &re, error, 255);
      NSDebugLog(@"error: %s\n", error);
      free(error);
    }

  return AUTORELEASE(aMutableArray);
}

@end
